#include <math.h>
#include "std.h"

#define EPS 1.e-10

/* fishtab -- returns percentile theta in degrees for the Fisher distribution   070299
	with a precision parameter k, at which the cumulative distribution h(theta) 
	is p, where h(th) is given by
		h(th) = \int_0^th \int_0^2pi f(th) sin(th) dth dphi
		      = (e^k - e^k*cos(th))/(2 shinh(k))
		      = (1 - e^k(cos(th)-1))/(1 - e^(-2k))
	and theta is determined by
		cos(th) = (1/k)*log((1-p)e^k + pe^-k)
			= 1 + (1/k)*log(1 - (1 - e^-2k)p)
	When k=0, the distribution is homogeneous and h(th) is given by
		h(th) = (1/2)*(1 - cos(th))
	and theta is determined by
		cos(th) = 1 - 2p
*/

double fishtab(k, p)
double k, p;
{
	double cost;

	if (k < 0.) error("Negative kappa!");
	if (p < 0. || p > 1.) error("p is out of range in fishtab!");
	if (k > EPS){
		cost = 1. + log(1. - (1. - exp(-2.*k))*p)/k;
		if (cost >= 1.)
			return 0.;
		else if (cost <= -1.)
			return 180.;
		else
			return RAD*acos(cost);
	}
	else
		return RAD*acos(1. - 2.*p);
}


/* fishcdist -- cumulative distribution of the Fisher distribution h(theta) for theta 
	which is in degrees
*/

double fishcdist(k, t)
double k, t;
{
	if (k < 0.) error("Negative kappa!");
	if (k > EPS)
		return ( 1. - exp(k*(cos(t/RAD) - 1.)) ) / ( 1. - exp(-2.*k) );
	else
		return (1. - cos(t/RAD))/2.;
}

/* fishdist -- density of Fisher distribution partly integrated by phi
		g(th) dth = (k/(2*sinh(k)) e^k*cos(th) sin(th) dth
*/

double fishdist(k, t)
double k, t;
{
	double tr;

	if (k < 0.) error("Negative kappa!");
	tr = t/RAD;
	if (k > EPS)
		return k*exp(k*(cos(tr) - 1.))*sin(tr)/( 1. - exp(-2.*k) );
	else
		return exp(k*cos(tr))*sin(tr)/2.;
}

/* fishdense -- probability density of Fisher distribution per unit area   060421
	probability of true direction included in small area dS is, 
		p(th) dS = (k/(4pi*sinh(k))e^k*cos(th) dS
*/

double fishdense(k, t)
double k, t;
{
	double tr;

	if (k < 0.) error("Negative kappa!");
	tr = t/RAD;
	if (k > EPS)
		return (k/DPAI)*exp(k*(cos(tr) - 1.))/(1. - exp(-2.*k));
	else
		return exp(k*cos(tr))/(2.*DPAI);
}
#undef EPS
