#include <math.h>
#include "std.h"

#define EPS1 0.001
#define EPS2 1.0e-8

/* kstestn -- Kolmogorov-Smirnov test for normal distribution   0400,0900,1218
	n,x: number of data stored in array x (x[0]-x[n-1])
	mean,stdev: mean & stdev of x which should be given by parent program
	D: K-S statistic D
	prob: significance level for the observed value of D
	Based on 14.3 of Numerical Recipies by Press et al. (1992)
	CAUTION: order of x[] will be changed due to sorting
*/

void kstestn(n, mean, stdev, x, D, prob)
int n;
double mean, stdev, *x, *D, *prob;
{
	int i;
	double en, Si, Pi, Di;

	sort(n, x);
	en = n;
	*D = 0.;
	for (i=0; i<n; i++){
		Si = (i + 1)/en;
		Pi = normcdist((x[i] - mean)/stdev);
		Di = fabs(Si - Pi);
		if (Di > *D) *D = Di;
	}
	en = sqrt(en);
	*prob = probks((en + 0.12 + 0.11/en)*(*D));
}

/* kstestf -- Kolmogorov-Smirnov test for Fisher distribution   0999,1218
	n   : number of array theta
	k   : Fisher's precision parameter
	th  : array (0 to n-1) of theta (co-inclination/angle distance) from the center
	dec : ditto of declination around the center
	Dt  : K-S statistics for th which is expected to follow Fisher distribution
	Dd  : ditto for dec which is expected to be uniform distribution
	prbt: significance level of th for the observed value of Dt
	prbd: ditto of dec for the observed value of Dd
	Calculation of (th,dec) from original (Inc,Dec) around center (Incc,Decc):
		rotuvec(90.-Decc, 90.-Incc, Inc, Dec, &dum1, &dum2);
		rotuvec(-90., 0., dum1, dum2, &Inct, &Dect);
		th = 90.-Inct;  dec = Dect;
	(see, "fldadf" & "fldadp in fldvgp")
	CAUTION: order of th[] & dec[] will be changed due to sorting
*/

void kstestf(n, k, th, dec, Dt, Dd, prbt, prbd)
int n;
double k, *th, *dec, *Dt, *Dd, *prbt, *prbd;
{
	int i;
	double en, Si, Pti, Pdi, Dti, Ddi;

	for (i=0; i<n; i++) if (dec[i] < 0.) dec[i] += 360.;
	sort(n, th);   sort(n, dec);
	en = n;
	*Dt = *Dd = 0.;
	for (i=0; i<n; i++){
		Si = (i + 1)/en;
		Pti = fishcdist(k, th[i]);
		Dti = fabs(Si - Pti);
		if (Dti > *Dt) *Dt = Dti;
		Pdi = dec[i]/360.;
		Ddi = fabs(Si - Pdi);
		if (Ddi > *Dd) *Dd = Ddi;
	}
	en = sqrt(en);
	*prbt = probks((en + 0.12 + 0.11/en)*(*Dt));
	*prbd = probks((en + 0.12 + 0.11/en)*(*Dd));
}

/* kstestl -- Kolmogorov-Smirnov test for log-normal distribution   041300
	n,x: number of data stored in array x (x[0]-x[n-1])
	ml,sl: mean & stdev of log(x) which should be given by parent program
	D: K-S statistics D
	prob: significance level for the observed value of D
	CAUTION: detection of negative x[] stops the program
		 order of array x[] will be changed due to sorting
*/

void kstestl(n, ml, sl, x, D, prob)
int n;
double ml, sl, *x, *D, *prob;
{
	int i, dumi;
	double en, Si, Pi, Di;

	sort(n, x);
	en = n;
	*D = 0.;
	for (i=0; i<n; i++){
		if (x[i] <= 0.) error("Negative data at kstestl!");
		Si = (i + 1)/en;
		Pi = normcdist((log(x[i]) - ml)/sl);
		Di = fabs(Si - Pi);
		if (Di > *D) *D = Di;
	}
	en = sqrt(en);
	*prob = probks((en + 0.12 + 0.11/en)*(*D));
}

/* kstest2 -- returns the Kolmogorov-Smirnov D for two dataset   081606
	n1,x1: number of 1st array data, x1[0]...x1[n1-1]
	n2,x2: 2nd dataset, x2[0]...x2[n2-1]
	D: K-S test statistic D
	prob: significance level for the observed value of D
	CAUTION: order of arrays x1[] & x2[] will be changed due to sorting
*/

void kstest2(n1, x1, n2, x2, D, prob)
int n1, n2;
double *x1, *x2, *D, *prob;
{
	int i1=0, i2=0;
	double en, en1, en2, Si1 = 0., Si2 = 0., Di, d1, d2;

	sort(n1, x1);   sort(n2, x2);
	en1 = n1;   en2 = n2;
	while (i1<n1 && i2<n2){
		if ((d1=x1[i1]) <= (d2=x2[i2])){
			Si1 = (i1 + 1)/en1;
			i1++;
		}
		if (d2 <= d1){
			Si2 = (i2 + 1)/en2;
			i2++;
		}
		Di = fabs(Si1 - Si2);
		if (Di > *D) *D = Di;
	}
	en = sqrt(en1*en2/(en1 + en2));
	*prob = probks((en + 0.12 + 0.11/en)*(*D));
}

/* probks -- Kolmogorov-Smirnov probability function (14.3.7) & (14.3.8) at
		p.624 of Numerical Recepies by Press et al.
*/

double probks(double alam)
{
	int j;
	double a2,fac=2.0,sum=0.0,term,termbf=0.0;

	a2 = -2.0*alam*alam;
	for (j=1;j<=100;j++) {
		term=fac*exp(a2*j*j);
		sum += term;
		if (fabs(term) <= EPS1*termbf || fabs(term) <= EPS2*sum) return sum;
		fac = -fac;
		termbf=fabs(term);
	}
	return 1.0;
}
#undef EPS1
#undef EPS2
/* (C) Copr. 1986-92 Numerical Recipes Software SD&5-X=!+!. */
/*
kstestn:
	printf("i,Si,Pi,Di: %d %6.4f %6.4f %6.4f\n", i, Si, Pi, Di);
kstestf:
	printf("i,Si,Pti,Dti,Pdi,Ddi: %d %6.4f %6.4f %6.4f %6.4f %6.4f\n",i,Si,Pti,Dti,Pdi,Ddi);
kstestl:
	printf("i,Si,Pi,Di: %d %6.4f %6.4f %6.4f\n", i, Si, Pi, Di);
kstest2:
	printf("i1,i2,Di: %d %d %f\n", i1, i2, Di);
	printf("en,prob: %f %g\n", en, *prob);
*/
