#include "std.h"

/* powell -- Powell's method of minimization of a n dimension function   0597,0297,0896
	minor update in memory allocation   0819
	Press et al. 1992, Numerical Recepies in C, p.417
	p: vector of starting point, later the best point. malloc as p[n+1]
	xi: matrix of initial directions, later current ones. malloc as xi[n+1][n+1]
		columns contain the direction vectors (usually unit vectors)
	ftol: fractional tolerence to stop (about square root of the machine epsilon)
	iter: returns number of iterations taken
	fret: returns the function value at p
	func: function to be found minimum, define as func(p) where p is a pointer to double!
	Returns ZERO when iteration exceeds ITMAX, otherwise returns +1.
*/

#include <math.h>

#define ITMAX 200
static double sqrarg;
#define SQR(a) ((sqrarg=(a)) == 0.0 ? 0.0 : sqrarg*sqrarg)

int powell(p, xi, n, ftol, iter, fret, func)
int n, *iter;
double *p, **xi, ftol, *fret, (*func)();
{
	int i,ibig,j;
	double del,fp,fptt,t,*pt,*ptt,*xit;

	pt = mallocvd(n+1);  ptt = mallocvd(n+1);  xit = mallocvd(n+1);
	*fret=(*func)(p);
	for (j=1;j<=n;j++) *(pt+j)=*(p+j);
	for (*iter=1;;++(*iter)) {
		fp=(*fret);
		ibig=0;
		del=0.0;
		for (i=1;i<=n;i++) {
			for (j=1;j<=n;j++) *(xit+j)=*(*xi+j*(n+1)+i);
			fptt=(*fret);
			linmin(p,xit,n,fret,func);
			if (fabs(fptt-(*fret)) > del) {
				del=fabs(fptt-(*fret));
				ibig=i;
			}
		}
		if (2.0*fabs(fp-(*fret)) <= ftol*(fabs(fp)+fabs(*fret))) {
			freevd(xit);  freevd(ptt);  freevd(pt);
			return(1);
		}
/*		if (*iter == ITMAX) error("powell: exceeds maximum iterations"); */
		if (*iter == ITMAX){
			freevd(xit);  freevd(ptt);  freevd(pt);
			return(ZERO);	/* over maximum iterations*/
		}
		for (j=1;j<=n;j++) {
			*(ptt+j)=2.0*(*(p+j))-*(pt+j);
			*(xit+j)=*(p+j)-*(pt+j);
			*(pt+j)=*(p+j);
		}
		fptt=(*func)(ptt);
		if (fptt < fp) {
			t=2.0*(fp-2.0*(*fret)+fptt)*SQR(fp-(*fret)-del)-del*SQR(fp-fptt);
			if (t < 0.0) {
				linmin(p,xit,n,fret,func);
				for (j=1;j<=n;j++) {
					*(*xi+j*(n+1)+ibig)=*(*xi+j*(n+1)+n);
					*(*xi+j*(n+1)+n)=*(xit+j);
				}
			}
		}
	}
	freevd(xit);  freevd(ptt);  freevd(pt);
	return(1);
}
#undef ITMAX
#undef SQR

/* linmin -- 1-d minimization used for powell()
	xx=-1.0 was set for better convergence to MLE of Bingham's params   0819
*/

#define TOL 2.0e-4

int ncom;
double *pcom, *xicom, (*nrfunc)();

void linmin(p, xi, n, fret, func)
int n;
double *p, *xi, *fret, (*func)();
{
	int j;
	double xx,xmin,fx,fb,fa,bx,ax,brent(),f1dim();

	ncom=n;
	pcom = mallocvd(n+1);   xicom = mallocvd(n+1);
	nrfunc=func;
	for (j=1;j<=n;j++) {
		*(pcom+j)=*(p+j);
		*(xicom+j)=*(xi+j);
	}
	ax=0.0;
/*	xx=1.0; */
	xx=-1.0;
	mnbrak(&ax,&xx,&bx,&fa,&fx,&fb,f1dim);
	*fret=brent(ax,xx,bx,f1dim,TOL,&xmin);
	for (j=1;j<=n;j++) {
		*(xi+j) *= xmin;
		*(p+j) += *(xi+j);
	}
	freevd(xicom);  freevd(pcom);
}

/* f1dim */

double f1dim(x)
double x;
{
	int j;
	double f,*xt;

	xt = mallocvd(ncom+1);
	for (j=1;j<=ncom;j++) *(xt+j)=*(pcom+j)+x*(*(xicom+j));
	f=(*nrfunc)(xt);
	freevd(xt);
	return f;
}
#undef TOL
