/* statb -- Bingham statistic of directions or poles   0597,1296
	Ver.1.2 -- Bingham's uniformity & circularity tests are added   0819
    Arguments:
	inc,dec: nd sets of (inc,dec) or (lat,lon).
		 ordinary nd-dimension array, and use as (0,1,2,...,nd-1)
	k1,k2: Bingham concentration parameters with k3=0
	tau1,tau2: 1st & 2nd eigenvalues, with 3rd, tau3=nd-tau1-tau2
	inct,dect: 3-d array of (inc,dec) or (lat,lon) of eigen vectors t1, t2, t3
		(inct[0],dect[0]) for the vector t1, and so on
	a95: 3-d array of 95% confidence radii
		 a95[0], a95[1], a95[2] give a31, a32, a21, respectively
    Variables:
	T,V: T matrix and a matrix of eigen vectors in columns.
	     4x4 matrixes, but use as (1,2,3)x(1,2,3)
    *** DO NOT USE "mallocmd" for memory alloc because jacobi's definition confilicts ***
	d: a vector of eigen values.
	   4-dimension vector, but use as (1,2,3)
	delta: 3-d array of Binghan's (7.22) Delta_ij multiplied by nd
		delta[0] = Delta_31 = (k3 - k1)*(tau3 - tau1)
		delta[1] = Delta_32 = (k3 - k2)*(tau3 - tau2)
		delta[3] = Delta_21 = (k2 - k1)*(tau2 - tau1)
    Returns:
	ZERO when error in jacobi() or mleb(), otherwise returns +1.
*/

#include <stdlib.h>
#include <math.h>
#include "std.h"

int statb(inc, dec, nd, k1, k2, tau1, tau2, inct, dect, a95)
int nd;
double *inc, *dec, *k1, *k2, *tau1, *tau2, *inct, *dect, *a95;
{
	int i, nrot, i1, i2, i3;
	double l, m, n, ll=0., lm=0., ln=0., mm=0., mn=0., nn=0.,
		**T, d[4], **V, cosi, sini, cosd, sind, delta[3],
		chi2 = 2.447747, dumx, dumy, dumz, dumm;

	if (nd < 3) error("Number of data should be N>=3!");
	if ( (T = (double **) malloc( (size_t)(4*sizeof(double*)) ) )
		== NULL ) error("error in memory allocation of T");
	if ( (*T = (double *) malloc( (size_t)(16*sizeof(double)) ) )
		== NULL ) error("error in memory allocation of *T");
	if ( (V = (double **) malloc( (size_t)(4*sizeof(double*)) ) )
		== NULL ) error("error in memory allocation of V");
	if ( (*V = (double *) malloc( (size_t)(16*sizeof(double)) ) )
		== NULL ) error("error in memory allocation of *V");
	for (i = 0; i < nd; i++){
		cosi = cos(inc[i]/RAD);   sini = sin(inc[i]/RAD);
		cosd = cos(dec[i]/RAD);   sind = sin(dec[i]/RAD);
		l = cosi*cosd;   m = cosi*sind;   n = sini;
		ll += l*l;   lm += l*m;   ln += l*n;
		mm += m*m;   mn += m*n;   nn += n*n;
	}
	*(*T+5) = ll;   *(*T+6) = lm;   *(*T+7) = ln;
	*(*T+9) = lm;   *(*T+10) = mm;   *(*T+11) = mn;
	*(*T+13) = ln;   *(*T+14) = mn;   *(*T+15) = nn;
	if (jacobi(T, 3, d, V, &nrot) == ZERO) return(ZERO);
	eigenorder(d, &i1, &i2, &i3);
	*tau1 = d[i1];   *tau2 = d[i2];
	if (mleb(nd, d[i1], d[i2], k1, k2) == ZERO) return(ZERO);
	delta[0] = -(*k1)*((double)nd - 2.*(*tau1) - *tau2);
	delta[1] = -(*k2)*((double)nd - *tau1 - 2.*(*tau2));
	delta[2] = (*k2 - *k1)*(*tau2 - *tau1);
	for (i = 0; i < 3; i++){
		if (delta[i] <= 0.)
			a95[i] = 360.;
		else 
			a95[i] = RAD*chi2*sqrt(0.5/delta[i]);
	}
	dumx = *(*V+4+i1);   dumy = *(*V+8+i1);   dumz = *(*V+12+i1);
	carmag(dumx, dumy, dumz, &dumm, inct, dect);
	dumx = *(*V+4+i2);   dumy = *(*V+8+i2);   dumz = *(*V+12+i2);
	carmag(dumx, dumy, dumz, &dumm, inct+1, dect+1);
	dumx = *(*V+4+i3);   dumy = *(*V+8+i3);   dumz = *(*V+12+i3);
	carmag(dumx, dumy, dumz, &dumm, inct+2, dect+2);
	free(T);   free(V);
	return(1);
}

/* chitest -- Bingham's test of data uniformity (Xu^2), polar circular symmetry (Xcp^2),
		girdle circularity (Xcg^2)   0819,6/98
	Null hypotheses of the distributions are rejected according to,
		Isotropy          : Xu^2 > Chi^2_0.95(5) = 11.070498
		Polar circularity : Xcp^2 > Chi^2_0.95(2) = 5.991465
		Girdle circularity: Xcg^2 > Chi^2_0.95(2) = 5.991465
*/

void chitest(n, t1, t2, k1, k2, Xu, Xcp, Xcg)
int n;
double t1, t2, k1, k2, *Xu, *Xcp, *Xcg;
{
	double dn, nb3, t3;

	dn = 2*n;   nb3 = (double)n/3.;   t3 = (double)n - t1 - t2;
	*Xu = ((t1 - nb3)*(t1 - nb3) + (t2 - nb3)*(t2 - nb3) + (t3 - nb3)*(t3 - nb3))*15./dn;
	*Xcp = .5*(t1 - t2)*(k1 - k2);
	*Xcg = .5*(t2 - t3)*k2;
}

/* memory of 4x4 matrix which is used to calculate 3x3 matrix
	 0   1   2   3
	 4   5   6   7
	 8   9  10  11
	12  13  14  15

printf("ll,lm,ln: %9.4e %9.4e %9.4e\n",ll,lm,ln);
printf("lm,mm,mn: %9.4e %9.4e %9.4e\n",lm,mm,mn);
printf("ln,mn,nn: %9.4e %9.4e %9.4e\n",ln,mn,nn);
*/
